@@ -1,5 +1,6 @@
 from typing import Iterable
 from sqlschm import sql, tok, lexer
+import dataclasses
 
 Lex = lexer.ItemCursor[tok.Token]
 
@@ -82,10 +83,12 @@ def _parse_column_def(l: Lex, /) -> tuple[sql.Column, list[sql.Constraint]]:
     colname = _parse_name(l)
     coltype = _parse_type(l)
     not_null = False
+    not_null_on_conflict: sql.OnConflict | None = None
     autoincrement = False
     generated = False
     default = None
     collation = None
+    pk: sql.Uniqueness | None = None
     col_constraints: list[sql.Constraint] = []
     while l.item is not tok.COMMA and l.item is not tok.R_PAREN:
         if l.item is tok.NULL:
@@ -93,13 +96,12 @@ def _parse_column_def(l: Lex, /) -> tuple[sql.Column, list[sql.Constraint]]:
         elif l.item is tok.NOT:
             l.forth()
             _expect(l, tok.NULL)
-            _parse_on_conflict(l)
+            not_null_on_conflict = _parse_on_conflict(l)
             not_null = True
         elif l.item is tok.DEFAULT:
             default = _parse_default(l)
         elif l.item is tok.COLLATE:
-            l.forth()
-            collation = _parse_name(l)
+            collation = _parse_collation(l)
         elif l.item is tok.GENERATED or l.item is tok.AS:
             if l.item is tok.GENERATED:
                 l.forth()
@@ -128,13 +130,23 @@ def _parse_column_def(l: Lex, /) -> tuple[sql.Column, list[sql.Constraint]]:
             autoincrement = True
         else:
             constraint = _parse_constraint(l, colname)
-            if constraint is not None:
+            if isinstance(constraint, sql.Uniqueness) and constraint.is_primary:
+                pk = constraint
+            elif constraint is not None:
                 col_constraints.append(constraint)
+    if pk is not None:
+        if collation is not None:
+            assert len(pk.indexed) == 1, "only one column"
+            pk = dataclasses.replace(
+                pk, indexed=[dataclasses.replace(pk.indexed[0], collation=collation)]
+            )
+        col_constraints.insert(0, pk)
     return (
         sql.Column(
             name=colname,
             type=coltype,
             not_null=not_null,
+            not_null_on_conflict=not_null_on_conflict,
             autoincrement=autoincrement,
             generated=generated,
             default=default,
@@ -201,6 +213,7 @@ def _parse_type(l: Lex, /) -> sql.Type:
 
 def _parse_constraint(l: Lex, col_name: str | None, /) -> sql.Constraint | None:
     columns = [] if col_name is None else [col_name]
+    indexed: list[sql.Indexed] = []
     name = None
     if l.item is tok.CONSTRAINT:
         l.forth()
@@ -208,23 +221,26 @@ def _parse_constraint(l: Lex, col_name: str | None, /) -> sql.Constraint | None:
     if l.item is tok.PRIMARY:
         l.forth()
         _expect(l, tok.KEY)
-        if l.item is tok.ASC or l.item is tok.DESC:
-            l.forth()
         if col_name is None:
-            columns = _parse_parens_names(l)
+            indexed = _parse_indexed_names(l)
+        else:
+            sorting = _parse_optional_sorting(l)
+            indexed = [sql.Indexed(column=col_name, sorting=sorting)]
         on_conflict = _parse_on_conflict(l)
         return sql.Uniqueness(
             name=name,
-            columns=columns,
+            indexed=indexed,
             is_primary=True,
             on_conflict=on_conflict,
         )
     elif l.item is tok.UNIQUE:
         l.forth()
         if col_name is None:
-            columns = _parse_parens_names(l)
+            indexed = _parse_indexed_names(l)
+        else:
+            indexed = [sql.Indexed(column=col_name)]
         on_conflict = _parse_on_conflict(l)
-        return sql.Uniqueness(name=name, columns=columns, on_conflict=on_conflict)
+        return sql.Uniqueness(name=name, indexed=indexed, on_conflict=on_conflict)
     elif l.item is tok.CHECK:
         l.forth()
         skip_parens(l)
@@ -240,14 +256,48 @@ def _parse_constraint(l: Lex, col_name: str | None, /) -> sql.Constraint | None:
         raise ParserError(f"'{l.item.val}' cannot start a constraint")
 
 
-def _parse_parens_names(l: Lex, /) -> list[str]:
+def _parse_collation(l: Lex, /) -> str:
+    _expect(l, tok.COLLATE)
+    return _parse_name(l)
+
+
+def _parse_indexed_names(l: Lex, /) -> list[sql.Indexed]:
     _expect(l, tok.L_PAREN)
-    names = [_parse_name(l)]
+    result = [_parse_indexed_name(l)]
     while l.item is tok.COMMA:
         l.forth()
-        names.append(_parse_name(l))
+        result.append(_parse_indexed_name(l))
     _expect(l, tok.R_PAREN)
-    return names
+    return result
+
+
+def _parse_indexed_name(l: Lex, /) -> sql.Indexed:
+    column = _parse_name(l)
+    collation = None
+    if l.item is tok.COLLATE:
+        collation = _parse_collation(l)
+    sorting = _parse_optional_sorting(l)
+    return sql.Indexed(column=column, collation=collation, sorting=sorting)
+
+
+def _parse_optional_sorting(l: Lex, /) -> sql.Sorting | None:
+    if l.item is tok.ASC:
+        l.forth()
+        return sql.Sorting.ASC
+    elif l.item is tok.DESC:
+        l.forth()
+        return sql.Sorting.DESC
+    return None
+
+
+def _parse_parens_names(l: Lex, /) -> list[str]:
+    _expect(l, tok.L_PAREN)
+    result = [_parse_name(l)]
+    while l.item is tok.COMMA:
+        l.forth()
+        result.append(_parse_name(l))
+    _expect(l, tok.R_PAREN)
+    return result
 
 
 def _parse_foreign_key_clause(
@@ -256,10 +306,9 @@ def _parse_foreign_key_clause(
     _expect(l, tok.REFERENCES)
     foreign_table = _parse_qualified_name(l)
     referred_columns: list[str] | None = None
-    on_delete = sql.OnUpdateDelete.NO_ACTION
-    on_update = sql.OnUpdateDelete.NO_ACTION
-    not_deferrable = False
-    initially_deferred = False
+    on_delete: sql.OnUpdateDelete | None = None
+    on_update: sql.OnUpdateDelete | None = None
+    match: sql.Match | None = None
     if l.item is tok.L_PAREN:
         referred_columns = _parse_parens_names(l)
     while l.item is tok.ON or l.item is tok.MATCH:
@@ -275,18 +324,11 @@ def _parse_foreign_key_clause(
                 raise ParserError("'ON DELETE' or 'ON UPDATE' is expected")
         elif l.item is tok.MATCH:
             l.forth()
-            l.forth()  # consume FULL or PARTIAL or SIMPLE
-    if l.item is tok.NOT and l.next_item is tok.DEFERRABLE:
-        l.forth()
-        not_deferrable = True
-    if l.item is tok.DEFERRABLE:
-        l.forth()
-        if l.item is tok.INITIALLY:
-            l.forth()
-            if not (l.item is tok.DEFERRED or l.item is tok.IMMEDIATE):
-                raise ParserError(f"'{l.item.val}' is not a valid dererrable state")
-            initially_deferred = l.item.val is tok.DEFERRED
-            l.forth()
+            match_name = _parse_name(l).upper()
+            if match_name not in sql.MATCH:
+                raise ParserError("invalid MATCH name")
+            match = sql.Match[match_name]
+    enforcement = _parse_constraint_enforcement(l)
     return sql.ForeignKey(
         name=name,
         columns=columns,
@@ -294,12 +336,45 @@ def _parse_foreign_key_clause(
         referred_columns=referred_columns,
         on_delete=on_delete,
         on_update=on_update,
-        enforcement=sql.ConstraintEnforcement(
-            initially_deferred=initially_deferred, not_deferrable=not_deferrable
-        ),
+        match=match,
+        enforcement=enforcement,
     )
 
 
+def _parse_constraint_enforcement(l: Lex, /) -> sql.ConstraintEnforcement | None:
+    initially = _parse_constraint_enforcement_time(l)
+    not_deferrable = None
+    if l.item is tok.NOT and l.next_item is tok.DEFERRABLE:
+        l.forth()
+        not_deferrable = True
+    if l.item is tok.DEFERRABLE:
+        l.forth()
+        not_deferrable = False
+        if initially is None:
+            initially = _parse_constraint_enforcement_time(l)
+    if not_deferrable is not None:
+        return sql.ConstraintEnforcement(
+            initially=initially, not_deferrable=not_deferrable
+        )
+    return None
+
+
+def _parse_constraint_enforcement_time(
+    l: Lex, /
+) -> sql.ConstraintEnforcementTime | None:
+    if l.item is tok.INITIALLY:
+        l.forth()
+        if l.item is tok.DEFERRED:
+            l.forth()
+            return sql.ConstraintEnforcementTime.DEFERRED
+        elif l.item is tok.IMMEDIATE:
+            l.forth()
+            return sql.ConstraintEnforcementTime.IMMEDIATE
+        else:
+            raise ParserError(f"'{l.item.val}' is not a valid enforcement time")
+    return None
+
+
 def _parse_on_updatedelete_action(l: Lex, /) -> sql.OnUpdateDelete:
     if l.item is tok.CASCADE:
         l.forth()
@@ -323,7 +398,7 @@ def _parse_on_updatedelete_action(l: Lex, /) -> sql.OnUpdateDelete:
         raise ParserError("'Invalid ON DELETE/UPDATE action")
 
 
-def _parse_on_conflict(l: Lex, /) -> sql.OnConflict:
+def _parse_on_conflict(l: Lex, /) -> sql.OnConflict | None:
     if l.item is tok.ON:
         l.forth()
         _expect(l, tok.CONFLICT)
@@ -333,7 +408,7 @@ def _parse_on_conflict(l: Lex, /) -> sql.OnConflict:
         l.forth()
         return result
     else:
-        return sql.OnConflict.ABORT  # Default
+        return None
 
 
 def _parse_int(l: Lex, /) -> int:
