@@ -13,6 +13,20 @@ class Dialect(_ReprEnum):
     SQLITE = auto()
 
 
+class ConstraintEnforcementTime(_ReprEnum):
+    DEFERRED = auto()
+    IMMEDIATE = auto()
+
+
+class Match(_ReprEnum):
+    FULL = auto()
+    PARTIAL = auto()
+    SIMPLE = auto()
+
+
+MATCH: frozenset[str] = frozenset(member.name for member in Match)
+
+
 # SQLite specific
 class OnConflict(_ReprEnum):
     ABORT = auto()
@@ -33,6 +47,11 @@ class OnUpdateDelete(_ReprEnum):
     SET_NULL = auto()
 
 
+class Sorting(_ReprEnum):
+    ASC = auto()
+    DESC = auto()
+
+
 """
 From the most specific to the least specific.
 e.g. database.schema.table is represented as ("table", "schema", "database")
@@ -56,6 +75,7 @@ class Column:
     name: str
     type: Type
     not_null: bool = False
+    not_null_on_conflict: OnConflict | None = None
     autoincrement: bool = False
     generated: bool = False
     default: Default | None = None
@@ -64,16 +84,23 @@ class Column:
 
 @dataclass(frozen=True, kw_only=True, slots=True)
 class ConstraintEnforcement:
-    initially_deferred: bool = False
+    initially: ConstraintEnforcementTime | None = None
     not_deferrable: bool = False
 
 
+@dataclass(frozen=True, kw_only=True, slots=True)
+class Indexed:
+    column: str
+    collation: str | None = None
+    sorting: Sorting | None = None
+
+
 @dataclass(frozen=True, kw_only=True, slots=True)
 class Uniqueness:
     name: str | None = None
-    columns: Sequence[str]
+    indexed: Sequence[Indexed]
     is_primary: bool = False
-    on_conflict: OnConflict = OnConflict.ABORT
+    on_conflict: OnConflict | None = None
 
 
 @dataclass(frozen=True, kw_only=True, slots=True)
@@ -82,9 +109,10 @@ class ForeignKey:
     columns: Sequence[str]
     foreign_table: QualifiedName
     referred_columns: Sequence[str] | None = None
-    on_delete: OnUpdateDelete = OnUpdateDelete.NO_ACTION
-    on_update: OnUpdateDelete = OnUpdateDelete.NO_ACTION
-    enforcement: ConstraintEnforcement
+    on_delete: OnUpdateDelete | None = None
+    on_update: OnUpdateDelete | None = None
+    match: Match | None = None
+    enforcement: ConstraintEnforcement | None = None
 
 
 Constraint = Uniqueness | ForeignKey
